pub struct PartiallyResolvedTriple {
pub subject: PossiblyResolved<String>,
pub predicate: PossiblyResolved<String>,
pub object: PossiblyResolved<ObjectType>,
}Expand description
A triple where the subject, predicate and object can all either be fully resolved to an id, or unresolved.
Fields
subject: PossiblyResolved<String>predicate: PossiblyResolved<String>object: PossiblyResolved<ObjectType>Implementations
Sourceimpl PartiallyResolvedTriple
impl PartiallyResolvedTriple
Sourcepub fn resolve_with(
&self,
node_map: &HashMap<String, u64>,
predicate_map: &HashMap<String, u64>,
value_map: &HashMap<TypedDictEntry, u64>,
) -> Option<IdTriple>
pub fn resolve_with( &self, node_map: &HashMap<String, u64>, predicate_map: &HashMap<String, u64>, value_map: &HashMap<TypedDictEntry, u64>, ) -> Option<IdTriple>
Resolve the unresolved ids in this triple using the given hashmaps for nodes, predicates and values.
pub fn as_resolved(&self) -> Option<IdTriple>
pub fn make_resolved_or_zero(&mut self)