CachedDirectoryLabelStore

Struct CachedDirectoryLabelStore 

pub struct CachedDirectoryLabelStore { /* private fields */ }
A version of the directory label store that keeps all labels in memory and doesn’t lock.

This is useful for situations where we can be sure that only one process is working with a set of label files. In that case, we can keep all label files cached in memory in order to process reads as quickly as possible, and we can perform writes without any sort of file system locking.

Implementations

Source

impl CachedDirectoryLabelStore

pub async fn open<P: Into<PathBuf>>(path: P) -> Result<Self>

Open a new label store.

This will read in all label files on startup, which is why this is an async operation.