NamedGraph

Struct NamedGraph 

pub struct NamedGraph { /* private fields */ }
A named graph in terminus-store.

Named graphs in terminus-store are basically just a label pointing to a layer. Opening a read transaction to a named graph is just getting hold of the layer it points at, as layers are read-only. Writing to a named graph is just making it point to a new layer.

Implementations

Source

impl NamedGraph

pub fn name(&self) -> &str

Returns the label name itself.

pub async fn head_version(&self) -> Result<(Option<StoreLayer>, u64)>

Returns the layer this database points at, as well as the label version.

pub async fn head(&self) -> Result<Option<StoreLayer>>

Returns the layer this database points at.

Examples found in repository?
examples/print_graph.rs (line 15)
8async fn print_graph(store_path: &str, graph: &str) -> io::Result<()> {
9    let store = open_directory_store(store_path);
10    let graph = store
11        .open(graph)
12        .await?
13        .expect(&format!("expected graph {} to exist", graph));
14
15    match graph.head().await? {
16        Some(layer) => {
17            for id_triple in layer.triples() {
18                // triples are retrieved in their id form. For printing,
19                // we need the string form. The conversion happens here.
20                let triple = layer
21                    .id_triple_to_string(&id_triple)
22                    .expect("expected id triple to be mapable to string");
23
24                println!(
25                    "{}, {}, {} {:?}",
26                    triple.subject,
27                    triple.predicate,
28                    match triple.object {
29                        ObjectType::Node(_) => "node",
30                        ObjectType::Value(_) => "value",
31                    },
32                    match triple.object {
33                        ObjectType::Node(n) => String::make_entry(&n),
34                        ObjectType::Value(v) => v,
35                    }
36                );
37            }
38        }
39        None => {}
40    }
41
42    Ok(())
43}
More examples
Hide additional examples
examples/write_to_graph.rs (line 64)
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55    let store = open_directory_store(store_path);
56    let graph = store
57        .open(graph)
58        .await?
59        .expect(&format!("expected graph {} to exist", graph));
60
61    // There are two types of builders. One creates a new base layer,
62    // which has no parent. The other creates a child layer, which has
63    // another layer as its parent.
64    let builder = match graph.head().await? {
65        Some(layer) => layer.open_write().await?,
66        None => store.create_base_layer().await?,
67    };
68    let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70    while let Some(line) = stdin.next_line().await? {
71        let segment = line.trim();
72        if segment.len() == 0 {
73            continue;
74        }
75
76        let command = parse_command(segment).await?;
77
78        // add all the input data into the builder.
79        // The builder keeps an in-memory list of added and removed
80        // triples. If the same triple is added and removed on the
81        // same builder, it is a no-op. This is even the case when it
82        // is then later re-added on the same builder.
83        //
84        // Since no io is happening, adding triples to the builder is
85        // not a future.
86        match command {
87            Command::Add(triple) => builder.add_value_triple(triple)?,
88            Command::Remove(triple) => builder.remove_value_triple(triple)?,
89        }
90    }
91
92    // When commit is called, the builder writes its data to
93    // persistent storage.
94    let layer = builder.commit().await?;
95
96    // While a layer exists now, it's not yet attached to anything,
97    // and is therefore unusable unless you know the exact identifier
98    // of the layer itself. To make this the graph data, we have to
99    // set the grap head to this layer.
100    graph.set_head(&layer).await?;
101
102    println!(
103        "Added: {}, removed: {}",
104        layer.triple_layer_addition_count().await?,
105        layer.triple_layer_removal_count().await?
106    );
107
108    Ok(())
109}
pub async fn set_head(&self, layer: &StoreLayer) -> Result<bool>

Set the database label to the given layer if it is a valid ancestor, returning false otherwise.

Examples found in repository?
examples/write_to_graph.rs (line 100)
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55    let store = open_directory_store(store_path);
56    let graph = store
57        .open(graph)
58        .await?
59        .expect(&format!("expected graph {} to exist", graph));
60
61    // There are two types of builders. One creates a new base layer,
62    // which has no parent. The other creates a child layer, which has
63    // another layer as its parent.
64    let builder = match graph.head().await? {
65        Some(layer) => layer.open_write().await?,
66        None => store.create_base_layer().await?,
67    };
68    let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70    while let Some(line) = stdin.next_line().await? {
71        let segment = line.trim();
72        if segment.len() == 0 {
73            continue;
74        }
75
76        let command = parse_command(segment).await?;
77
78        // add all the input data into the builder.
79        // The builder keeps an in-memory list of added and removed
80        // triples. If the same triple is added and removed on the
81        // same builder, it is a no-op. This is even the case when it
82        // is then later re-added on the same builder.
83        //
84        // Since no io is happening, adding triples to the builder is
85        // not a future.
86        match command {
87            Command::Add(triple) => builder.add_value_triple(triple)?,
88            Command::Remove(triple) => builder.remove_value_triple(triple)?,
89        }
90    }
91
92    // When commit is called, the builder writes its data to
93    // persistent storage.
94    let layer = builder.commit().await?;
95
96    // While a layer exists now, it's not yet attached to anything,
97    // and is therefore unusable unless you know the exact identifier
98    // of the layer itself. To make this the graph data, we have to
99    // set the grap head to this layer.
100    graph.set_head(&layer).await?;
101
102    println!(
103        "Added: {}, removed: {}",
104        layer.triple_layer_addition_count().await?,
105        layer.triple_layer_removal_count().await?
106    );
107
108    Ok(())
109}
pub async fn force_set_head(&self, layer: &StoreLayer) -> Result<()>

Set the database label to the given layer, even if it is not a valid ancestor.

pub async fn force_set_head_version( &self, layer: &StoreLayer, version: u64, ) -> Result<bool>

Set the database label to the given layer, even if it is not a valid ancestor. Also checks given version, and if it doesn’t match, the update won’t happen and false will be returned.

pub async fn delete(&self) -> Result<()>