StoreLayer

Struct StoreLayer 

pub struct StoreLayer { /* private fields */ }
A layer that keeps track of the store it came out of, allowing the creation of a layer builder on top of this layer.

This type of layer supports querying what was added and what was removed in this layer. This can not be done in general, because the layer that has been loaded may not be the layer that was originally built. This happens whenever a rollup is done. A rollup will create a new layer that bundles the changes of various layers. It allows for more efficient querying, but loses the ability to do these delta queries directly. In order to support them anyway, the StoreLayer will dynamically load in the relevant files to perform the requested addition or removal query method.

Implementations

Source

impl StoreLayer

pub async fn open_write(&self) -> Result<StoreLayerBuilder>

Create a layer builder based on this layer.

Examples found in repository?
examples/write_to_graph.rs (line 65)
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55    let store = open_directory_store(store_path);
56    let graph = store
57        .open(graph)
58        .await?
59        .expect(&format!("expected graph {} to exist", graph));
60
61    // There are two types of builders. One creates a new base layer,
62    // which has no parent. The other creates a child layer, which has
63    // another layer as its parent.
64    let builder = match graph.head().await? {
65        Some(layer) => layer.open_write().await?,
66        None => store.create_base_layer().await?,
67    };
68    let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70    while let Some(line) = stdin.next_line().await? {
71        let segment = line.trim();
72        if segment.len() == 0 {
73            continue;
74        }
75
76        let command = parse_command(segment).await?;
77
78        // add all the input data into the builder.
79        // The builder keeps an in-memory list of added and removed
80        // triples. If the same triple is added and removed on the
81        // same builder, it is a no-op. This is even the case when it
82        // is then later re-added on the same builder.
83        //
84        // Since no io is happening, adding triples to the builder is
85        // not a future.
86        match command {
87            Command::Add(triple) => builder.add_value_triple(triple)?,
88            Command::Remove(triple) => builder.remove_value_triple(triple)?,
89        }
90    }
91
92    // When commit is called, the builder writes its data to
93    // persistent storage.
94    let layer = builder.commit().await?;
95
96    // While a layer exists now, it's not yet attached to anything,
97    // and is therefore unusable unless you know the exact identifier
98    // of the layer itself. To make this the graph data, we have to
99    // set the grap head to this layer.
100    graph.set_head(&layer).await?;
101
102    println!(
103        "Added: {}, removed: {}",
104        layer.triple_layer_addition_count().await?,
105        layer.triple_layer_removal_count().await?
106    );
107
108    Ok(())
109}
pub async fn parent(&self) -> Result<Option<StoreLayer>>

Returns the parent of this layer, if any, or None if this layer has no parent.

pub async fn squash_upto(&self, upto: &StoreLayer) -> Result<StoreLayer>

pub async fn squash(&self) -> Result<StoreLayer>

Create a new base layer consisting of all triples in this layer, as well as all its ancestors.

It is a good idea to keep layer stacks small, meaning, to only have a handful of ancestors for a layer. The more layers there are, the longer queries take. Squash is one approach of accomplishing this. Rollup is another. Squash is the better option if you do not care for history, as it throws away all data that you no longer need.

pub async fn rollup(&self) -> Result<()>

Create a new rollup layer which rolls up all triples in this layer, as well as all its ancestors.

It is a good idea to keep layer stacks small, meaning, to only have a handful of ancestors for a layer. The more layers there are, the longer queries take. Rollup is one approach of accomplishing this. Squash is another. Rollup is the better option if you need to retain history.

pub async fn rollup_upto(&self, upto: &StoreLayer) -> Result<()>

Create a new rollup layer which rolls up all triples in this layer, as well as all ancestors up to (but not including) the given ancestor.

It is a good idea to keep layer stacks small, meaning, to only have a handful of ancestors for a layer. The more layers there are, the longer queries take. Rollup is one approach of accomplishing this. Squash is another. Rollup is the better option if you need to retain history.

pub async fn imprecise_rollup_upto(&self, upto: &StoreLayer) -> Result<()>

Like rollup_upto, rolls up upto the given layer. However, if this layer is a rollup layer, this will roll up upto that rollup.

pub async fn triple_addition_exists( &self, subject: u64, predicate: u64, object: u64, ) -> Result<bool>

Returns a future that yields true if this triple has been added in this layer, or false if it doesn’t.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_removal_exists( &self, subject: u64, predicate: u64, object: u64, ) -> Result<bool>

Returns a future that yields true if this triple has been removed in this layer, or false if it doesn’t.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_additions( &self, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>

Returns a future that yields an iterator over all layer additions.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_removals( &self, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>

Returns a future that yields an iterator over all layer removals.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_additions_s( &self, subject: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>

Returns a future that yields an iterator over all layer additions that share a particular subject.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_removals_s( &self, subject: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>

Returns a future that yields an iterator over all layer removals that share a particular subject.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_additions_sp( &self, subject: u64, predicate: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>

Returns a future that yields an iterator over all layer additions that share a particular subject and predicate.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_removals_sp( &self, subject: u64, predicate: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>

Returns a future that yields an iterator over all layer removals that share a particular subject and predicate.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_additions_p( &self, predicate: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>

Returns a future that yields an iterator over all layer additions that share a particular predicate.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_removals_p( &self, predicate: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>

Returns a future that yields an iterator over all layer removals that share a particular predicate.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_additions_o( &self, object: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>

Returns a future that yields an iterator over all layer additions that share a particular object.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_removals_o( &self, object: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>

Returns a future that yields an iterator over all layer removals that share a particular object.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

pub async fn triple_layer_addition_count(&self) -> Result<usize>

Returns a future that yields the amount of triples that this layer adds.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

Examples found in repository?
examples/write_to_graph.rs (line 104)
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55    let store = open_directory_store(store_path);
56    let graph = store
57        .open(graph)
58        .await?
59        .expect(&format!("expected graph {} to exist", graph));
60
61    // There are two types of builders. One creates a new base layer,
62    // which has no parent. The other creates a child layer, which has
63    // another layer as its parent.
64    let builder = match graph.head().await? {
65        Some(layer) => layer.open_write().await?,
66        None => store.create_base_layer().await?,
67    };
68    let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70    while let Some(line) = stdin.next_line().await? {
71        let segment = line.trim();
72        if segment.len() == 0 {
73            continue;
74        }
75
76        let command = parse_command(segment).await?;
77
78        // add all the input data into the builder.
79        // The builder keeps an in-memory list of added and removed
80        // triples. If the same triple is added and removed on the
81        // same builder, it is a no-op. This is even the case when it
82        // is then later re-added on the same builder.
83        //
84        // Since no io is happening, adding triples to the builder is
85        // not a future.
86        match command {
87            Command::Add(triple) => builder.add_value_triple(triple)?,
88            Command::Remove(triple) => builder.remove_value_triple(triple)?,
89        }
90    }
91
92    // When commit is called, the builder writes its data to
93    // persistent storage.
94    let layer = builder.commit().await?;
95
96    // While a layer exists now, it's not yet attached to anything,
97    // and is therefore unusable unless you know the exact identifier
98    // of the layer itself. To make this the graph data, we have to
99    // set the grap head to this layer.
100    graph.set_head(&layer).await?;
101
102    println!(
103        "Added: {}, removed: {}",
104        layer.triple_layer_addition_count().await?,
105        layer.triple_layer_removal_count().await?
106    );
107
108    Ok(())
109}
pub async fn triple_layer_removal_count(&self) -> Result<usize>

Returns a future that yields the amount of triples that this layer removes.

Since this operation will involve io when this layer is a rollup layer, io errors may occur.

Examples found in repository?
examples/write_to_graph.rs (line 105)
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55    let store = open_directory_store(store_path);
56    let graph = store
57        .open(graph)
58        .await?
59        .expect(&format!("expected graph {} to exist", graph));
60
61    // There are two types of builders. One creates a new base layer,
62    // which has no parent. The other creates a child layer, which has
63    // another layer as its parent.
64    let builder = match graph.head().await? {
65        Some(layer) => layer.open_write().await?,
66        None => store.create_base_layer().await?,
67    };
68    let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70    while let Some(line) = stdin.next_line().await? {
71        let segment = line.trim();
72        if segment.len() == 0 {
73            continue;
74        }
75
76        let command = parse_command(segment).await?;
77
78        // add all the input data into the builder.
79        // The builder keeps an in-memory list of added and removed
80        // triples. If the same triple is added and removed on the
81        // same builder, it is a no-op. This is even the case when it
82        // is then later re-added on the same builder.
83        //
84        // Since no io is happening, adding triples to the builder is
85        // not a future.
86        match command {
87            Command::Add(triple) => builder.add_value_triple(triple)?,
88            Command::Remove(triple) => builder.remove_value_triple(triple)?,
89        }
90    }
91
92    // When commit is called, the builder writes its data to
93    // persistent storage.
94    let layer = builder.commit().await?;
95
96    // While a layer exists now, it's not yet attached to anything,
97    // and is therefore unusable unless you know the exact identifier
98    // of the layer itself. To make this the graph data, we have to
99    // set the grap head to this layer.
100    graph.set_head(&layer).await?;
101
102    println!(
103        "Added: {}, removed: {}",
104        layer.triple_layer_addition_count().await?,
105        layer.triple_layer_removal_count().await?
106    );
107
108    Ok(())
109}
pub async fn retrieve_layer_stack_names(&self) -> Result<Vec<[u32; 5]>>

Returns a future that yields a vector of layer stack names describing the history of this layer, starting from the base layer up to and including the name of this layer itself.