pub struct StoreLayerBuilder { /* private fields */ }Expand description
A wrapper over a SimpleLayerBuilder, providing a thread-safe sharable interface.
The SimpleLayerBuilder requires one to have a mutable reference to the underlying LayerBuilder, and on commit it will be consumed. This builder only requires an immutable reference, and uses a futures-aware read-write lock to synchronize access to it between threads. Also, rather than consuming itself on commit, this wrapper will simply mark itself as having committed, returning errors on further calls.
Implementations
Sourceimpl StoreLayerBuilder
impl StoreLayerBuilder
pub fn with_builder<R, F: FnOnce(&mut Box<dyn LayerBuilder>) -> R>( &self, f: F, ) -> Result<R, Error>
Sourcepub fn parent(&self) -> Option<Arc<dyn Layer>>
pub fn parent(&self) -> Option<Arc<dyn Layer>>
Returns the parent layer this builder is building on top of, if any.
If there’s no parent, this returns None.
Sourcepub fn add_value_triple(&self, triple: ValueTriple) -> Result<(), Error>
pub fn add_value_triple(&self, triple: ValueTriple) -> Result<(), Error>
Add a string triple.
Examples found in repository?
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55 let store = open_directory_store(store_path);
56 let graph = store
57 .open(graph)
58 .await?
59 .expect(&format!("expected graph {} to exist", graph));
60
61 // There are two types of builders. One creates a new base layer,
62 // which has no parent. The other creates a child layer, which has
63 // another layer as its parent.
64 let builder = match graph.head().await? {
65 Some(layer) => layer.open_write().await?,
66 None => store.create_base_layer().await?,
67 };
68 let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70 while let Some(line) = stdin.next_line().await? {
71 let segment = line.trim();
72 if segment.len() == 0 {
73 continue;
74 }
75
76 let command = parse_command(segment).await?;
77
78 // add all the input data into the builder.
79 // The builder keeps an in-memory list of added and removed
80 // triples. If the same triple is added and removed on the
81 // same builder, it is a no-op. This is even the case when it
82 // is then later re-added on the same builder.
83 //
84 // Since no io is happening, adding triples to the builder is
85 // not a future.
86 match command {
87 Command::Add(triple) => builder.add_value_triple(triple)?,
88 Command::Remove(triple) => builder.remove_value_triple(triple)?,
89 }
90 }
91
92 // When commit is called, the builder writes its data to
93 // persistent storage.
94 let layer = builder.commit().await?;
95
96 // While a layer exists now, it's not yet attached to anything,
97 // and is therefore unusable unless you know the exact identifier
98 // of the layer itself. To make this the graph data, we have to
99 // set the grap head to this layer.
100 graph.set_head(&layer).await?;
101
102 println!(
103 "Added: {}, removed: {}",
104 layer.triple_layer_addition_count().await?,
105 layer.triple_layer_removal_count().await?
106 );
107
108 Ok(())
109}Sourcepub fn remove_value_triple(&self, triple: ValueTriple) -> Result<(), Error>
pub fn remove_value_triple(&self, triple: ValueTriple) -> Result<(), Error>
Remove a string triple.
Examples found in repository?
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55 let store = open_directory_store(store_path);
56 let graph = store
57 .open(graph)
58 .await?
59 .expect(&format!("expected graph {} to exist", graph));
60
61 // There are two types of builders. One creates a new base layer,
62 // which has no parent. The other creates a child layer, which has
63 // another layer as its parent.
64 let builder = match graph.head().await? {
65 Some(layer) => layer.open_write().await?,
66 None => store.create_base_layer().await?,
67 };
68 let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70 while let Some(line) = stdin.next_line().await? {
71 let segment = line.trim();
72 if segment.len() == 0 {
73 continue;
74 }
75
76 let command = parse_command(segment).await?;
77
78 // add all the input data into the builder.
79 // The builder keeps an in-memory list of added and removed
80 // triples. If the same triple is added and removed on the
81 // same builder, it is a no-op. This is even the case when it
82 // is then later re-added on the same builder.
83 //
84 // Since no io is happening, adding triples to the builder is
85 // not a future.
86 match command {
87 Command::Add(triple) => builder.add_value_triple(triple)?,
88 Command::Remove(triple) => builder.remove_value_triple(triple)?,
89 }
90 }
91
92 // When commit is called, the builder writes its data to
93 // persistent storage.
94 let layer = builder.commit().await?;
95
96 // While a layer exists now, it's not yet attached to anything,
97 // and is therefore unusable unless you know the exact identifier
98 // of the layer itself. To make this the graph data, we have to
99 // set the grap head to this layer.
100 graph.set_head(&layer).await?;
101
102 println!(
103 "Added: {}, removed: {}",
104 layer.triple_layer_addition_count().await?,
105 layer.triple_layer_removal_count().await?
106 );
107
108 Ok(())
109}Sourcepub fn committed(&self) -> bool
pub fn committed(&self) -> bool
Returns true if this layer has been committed, and false otherwise.
Sourcepub async fn commit_no_load(&self) -> Result<()>
pub async fn commit_no_load(&self) -> Result<()>
Commit the layer to storage without loading the resulting layer.
Sourcepub async fn commit(&self) -> Result<StoreLayer>
pub async fn commit(&self) -> Result<StoreLayer>
Commit the layer to storage.
Examples found in repository?
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55 let store = open_directory_store(store_path);
56 let graph = store
57 .open(graph)
58 .await?
59 .expect(&format!("expected graph {} to exist", graph));
60
61 // There are two types of builders. One creates a new base layer,
62 // which has no parent. The other creates a child layer, which has
63 // another layer as its parent.
64 let builder = match graph.head().await? {
65 Some(layer) => layer.open_write().await?,
66 None => store.create_base_layer().await?,
67 };
68 let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70 while let Some(line) = stdin.next_line().await? {
71 let segment = line.trim();
72 if segment.len() == 0 {
73 continue;
74 }
75
76 let command = parse_command(segment).await?;
77
78 // add all the input data into the builder.
79 // The builder keeps an in-memory list of added and removed
80 // triples. If the same triple is added and removed on the
81 // same builder, it is a no-op. This is even the case when it
82 // is then later re-added on the same builder.
83 //
84 // Since no io is happening, adding triples to the builder is
85 // not a future.
86 match command {
87 Command::Add(triple) => builder.add_value_triple(triple)?,
88 Command::Remove(triple) => builder.remove_value_triple(triple)?,
89 }
90 }
91
92 // When commit is called, the builder writes its data to
93 // persistent storage.
94 let layer = builder.commit().await?;
95
96 // While a layer exists now, it's not yet attached to anything,
97 // and is therefore unusable unless you know the exact identifier
98 // of the layer itself. To make this the graph data, we have to
99 // set the grap head to this layer.
100 graph.set_head(&layer).await?;
101
102 println!(
103 "Added: {}, removed: {}",
104 layer.triple_layer_addition_count().await?,
105 layer.triple_layer_removal_count().await?
106 );
107
108 Ok(())
109}Sourcepub async fn apply_delta(&self, delta: &StoreLayer) -> Result<(), Error>
pub async fn apply_delta(&self, delta: &StoreLayer) -> Result<(), Error>
Apply all triples added and removed by a layer to this builder.
This is a way to ‘cherry-pick’ a layer on top of another layer, without caring about its history.
Sourcepub fn apply_diff(&self, other: &StoreLayer) -> Result<(), Error>
pub fn apply_diff(&self, other: &StoreLayer) -> Result<(), Error>
Apply the changes required to change our parent layer into the given layer.