pub struct SyncNamedGraph { /* private fields */ }Expand description
A named graph in terminus-store.
Named graphs in terminus-store are basically just a label pointing to a layer. Opening a read transaction to a named graph is just getting hold of the layer it points at, as layers are read-only. Writing to a named graph is just making it point to a new layer.
Implementations
Sourceimpl SyncNamedGraph
impl SyncNamedGraph
Sourcepub fn head_version(&self) -> Result<(Option<SyncStoreLayer>, u64)>
pub fn head_version(&self) -> Result<(Option<SyncStoreLayer>, u64)>
Returns the layer this database points at, as well as the label version.
Sourcepub fn head(&self) -> Result<Option<SyncStoreLayer>>
pub fn head(&self) -> Result<Option<SyncStoreLayer>>
Returns the layer this database points at.
Sourcepub fn set_head(&self, layer: &SyncStoreLayer) -> Result<bool, Error>
pub fn set_head(&self, layer: &SyncStoreLayer) -> Result<bool, Error>
Set the database label to the given layer if it is a valid ancestor, returning false otherwise.
Sourcepub fn force_set_head(&self, layer: &SyncStoreLayer) -> Result<(), Error>
pub fn force_set_head(&self, layer: &SyncStoreLayer) -> Result<(), Error>
Set the database label to the given layer, even if it is not a valid ancestor.
Sourcepub fn force_set_head_version(
&self,
layer: &SyncStoreLayer,
version: u64,
) -> Result<bool>
pub fn force_set_head_version( &self, layer: &SyncStoreLayer, version: u64, ) -> Result<bool>
Set the database label to the given layer, even if it is not a valid ancestor. Also checks given version, and if it doesn’t match, the update won’t happen and false will be returned.
pub fn delete(&self) -> Result<()>