Enum terminus_store::layer::PossiblyResolved
source · pub enum PossiblyResolved<T: Clone + PartialEq + Eq + PartialOrd + Ord + Hash> {
Unresolved(T),
Resolved(u64),
}Expand description
Either a resolved id or an unresolved inner type.
Variants
Implementations
sourceimpl<T: Clone + PartialEq + Eq + PartialOrd + Ord + Hash> PossiblyResolved<T>
impl<T: Clone + PartialEq + Eq + PartialOrd + Ord + Hash> PossiblyResolved<T>
sourcepub fn is_resolved(&self) -> bool
pub fn is_resolved(&self) -> bool
Returns true if this is a resolved id, and false otherwise.
sourcepub fn as_ref(&self) -> PossiblyResolved<&T>
pub fn as_ref(&self) -> PossiblyResolved<&T>
Return a PossiblyResolved with the inner value as a reference.
sourcepub fn unwrap_unresolved(self) -> T
pub fn unwrap_unresolved(self) -> T
Unwrap to the unresolved inner value, or panic if this was actually a resolved id.
sourcepub fn unwrap_resolved(self) -> u64
pub fn unwrap_resolved(self) -> u64
Unwrap to the resolved id, or panic if this was actually an unresolved value.