pub struct StoreLayer { /* private fields */ }Expand description
A layer that keeps track of the store it came out of, allowing the creation of a layer builder on top of this layer.
This type of layer supports querying what was added and what was removed in this layer. This can not be done in general, because the layer that has been loaded may not be the layer that was originally built. This happens whenever a rollup is done. A rollup will create a new layer that bundles the changes of various layers. It allows for more efficient querying, but loses the ability to do these delta queries directly. In order to support them anyway, the StoreLayer will dynamically load in the relevant files to perform the requested addition or removal query method.
Implementations
Sourceimpl StoreLayer
impl StoreLayer
Sourcepub async fn open_write(&self) -> Result<StoreLayerBuilder>
pub async fn open_write(&self) -> Result<StoreLayerBuilder>
Create a layer builder based on this layer.
Examples found in repository?
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55 let store = open_directory_store(store_path);
56 let graph = store
57 .open(graph)
58 .await?
59 .expect(&format!("expected graph {} to exist", graph));
60
61 // There are two types of builders. One creates a new base layer,
62 // which has no parent. The other creates a child layer, which has
63 // another layer as its parent.
64 let builder = match graph.head().await? {
65 Some(layer) => layer.open_write().await?,
66 None => store.create_base_layer().await?,
67 };
68 let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70 while let Some(line) = stdin.next_line().await? {
71 let segment = line.trim();
72 if segment.len() == 0 {
73 continue;
74 }
75
76 let command = parse_command(segment).await?;
77
78 // add all the input data into the builder.
79 // The builder keeps an in-memory list of added and removed
80 // triples. If the same triple is added and removed on the
81 // same builder, it is a no-op. This is even the case when it
82 // is then later re-added on the same builder.
83 //
84 // Since no io is happening, adding triples to the builder is
85 // not a future.
86 match command {
87 Command::Add(triple) => builder.add_value_triple(triple)?,
88 Command::Remove(triple) => builder.remove_value_triple(triple)?,
89 }
90 }
91
92 // When commit is called, the builder writes its data to
93 // persistent storage.
94 let layer = builder.commit().await?;
95
96 // While a layer exists now, it's not yet attached to anything,
97 // and is therefore unusable unless you know the exact identifier
98 // of the layer itself. To make this the graph data, we have to
99 // set the grap head to this layer.
100 graph.set_head(&layer).await?;
101
102 println!(
103 "Added: {}, removed: {}",
104 layer.triple_layer_addition_count().await?,
105 layer.triple_layer_removal_count().await?
106 );
107
108 Ok(())
109}Sourcepub async fn parent(&self) -> Result<Option<StoreLayer>>
pub async fn parent(&self) -> Result<Option<StoreLayer>>
Returns the parent of this layer, if any, or None if this layer has no parent.
pub async fn squash_upto(&self, upto: &StoreLayer) -> Result<StoreLayer>
Sourcepub async fn squash(&self) -> Result<StoreLayer>
pub async fn squash(&self) -> Result<StoreLayer>
Create a new base layer consisting of all triples in this layer, as well as all its ancestors.
It is a good idea to keep layer stacks small, meaning, to only have a handful of ancestors for a layer. The more layers there are, the longer queries take. Squash is one approach of accomplishing this. Rollup is another. Squash is the better option if you do not care for history, as it throws away all data that you no longer need.
Sourcepub async fn rollup(&self) -> Result<()>
pub async fn rollup(&self) -> Result<()>
Create a new rollup layer which rolls up all triples in this layer, as well as all its ancestors.
It is a good idea to keep layer stacks small, meaning, to only have a handful of ancestors for a layer. The more layers there are, the longer queries take. Rollup is one approach of accomplishing this. Squash is another. Rollup is the better option if you need to retain history.
Sourcepub async fn rollup_upto(&self, upto: &StoreLayer) -> Result<()>
pub async fn rollup_upto(&self, upto: &StoreLayer) -> Result<()>
Create a new rollup layer which rolls up all triples in this layer, as well as all ancestors up to (but not including) the given ancestor.
It is a good idea to keep layer stacks small, meaning, to only have a handful of ancestors for a layer. The more layers there are, the longer queries take. Rollup is one approach of accomplishing this. Squash is another. Rollup is the better option if you need to retain history.
Sourcepub async fn imprecise_rollup_upto(&self, upto: &StoreLayer) -> Result<()>
pub async fn imprecise_rollup_upto(&self, upto: &StoreLayer) -> Result<()>
Like rollup_upto, rolls up upto the given layer. However, if this layer is a rollup layer, this will roll up upto that rollup.
Sourcepub async fn triple_addition_exists(
&self,
subject: u64,
predicate: u64,
object: u64,
) -> Result<bool>
pub async fn triple_addition_exists( &self, subject: u64, predicate: u64, object: u64, ) -> Result<bool>
Returns a future that yields true if this triple has been added in this layer, or false if it doesn’t.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_removal_exists(
&self,
subject: u64,
predicate: u64,
object: u64,
) -> Result<bool>
pub async fn triple_removal_exists( &self, subject: u64, predicate: u64, object: u64, ) -> Result<bool>
Returns a future that yields true if this triple has been removed in this layer, or false if it doesn’t.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_additions(
&self,
) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
pub async fn triple_additions( &self, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
Returns a future that yields an iterator over all layer additions.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_removals(
&self,
) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
pub async fn triple_removals( &self, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
Returns a future that yields an iterator over all layer removals.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_additions_s(
&self,
subject: u64,
) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
pub async fn triple_additions_s( &self, subject: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
Returns a future that yields an iterator over all layer additions that share a particular subject.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_removals_s(
&self,
subject: u64,
) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
pub async fn triple_removals_s( &self, subject: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
Returns a future that yields an iterator over all layer removals that share a particular subject.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_additions_sp(
&self,
subject: u64,
predicate: u64,
) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
pub async fn triple_additions_sp( &self, subject: u64, predicate: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
Returns a future that yields an iterator over all layer additions that share a particular subject and predicate.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_removals_sp(
&self,
subject: u64,
predicate: u64,
) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
pub async fn triple_removals_sp( &self, subject: u64, predicate: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
Returns a future that yields an iterator over all layer removals that share a particular subject and predicate.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_additions_p(
&self,
predicate: u64,
) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
pub async fn triple_additions_p( &self, predicate: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
Returns a future that yields an iterator over all layer additions that share a particular predicate.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_removals_p(
&self,
predicate: u64,
) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
pub async fn triple_removals_p( &self, predicate: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
Returns a future that yields an iterator over all layer removals that share a particular predicate.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_additions_o(
&self,
object: u64,
) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
pub async fn triple_additions_o( &self, object: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
Returns a future that yields an iterator over all layer additions that share a particular object.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_removals_o(
&self,
object: u64,
) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
pub async fn triple_removals_o( &self, object: u64, ) -> Result<Box<dyn Iterator<Item = IdTriple> + Send>>
Returns a future that yields an iterator over all layer removals that share a particular object.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Sourcepub async fn triple_layer_addition_count(&self) -> Result<usize>
pub async fn triple_layer_addition_count(&self) -> Result<usize>
Returns a future that yields the amount of triples that this layer adds.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Examples found in repository?
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55 let store = open_directory_store(store_path);
56 let graph = store
57 .open(graph)
58 .await?
59 .expect(&format!("expected graph {} to exist", graph));
60
61 // There are two types of builders. One creates a new base layer,
62 // which has no parent. The other creates a child layer, which has
63 // another layer as its parent.
64 let builder = match graph.head().await? {
65 Some(layer) => layer.open_write().await?,
66 None => store.create_base_layer().await?,
67 };
68 let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70 while let Some(line) = stdin.next_line().await? {
71 let segment = line.trim();
72 if segment.len() == 0 {
73 continue;
74 }
75
76 let command = parse_command(segment).await?;
77
78 // add all the input data into the builder.
79 // The builder keeps an in-memory list of added and removed
80 // triples. If the same triple is added and removed on the
81 // same builder, it is a no-op. This is even the case when it
82 // is then later re-added on the same builder.
83 //
84 // Since no io is happening, adding triples to the builder is
85 // not a future.
86 match command {
87 Command::Add(triple) => builder.add_value_triple(triple)?,
88 Command::Remove(triple) => builder.remove_value_triple(triple)?,
89 }
90 }
91
92 // When commit is called, the builder writes its data to
93 // persistent storage.
94 let layer = builder.commit().await?;
95
96 // While a layer exists now, it's not yet attached to anything,
97 // and is therefore unusable unless you know the exact identifier
98 // of the layer itself. To make this the graph data, we have to
99 // set the grap head to this layer.
100 graph.set_head(&layer).await?;
101
102 println!(
103 "Added: {}, removed: {}",
104 layer.triple_layer_addition_count().await?,
105 layer.triple_layer_removal_count().await?
106 );
107
108 Ok(())
109}Sourcepub async fn triple_layer_removal_count(&self) -> Result<usize>
pub async fn triple_layer_removal_count(&self) -> Result<usize>
Returns a future that yields the amount of triples that this layer removes.
Since this operation will involve io when this layer is a rollup layer, io errors may occur.
Examples found in repository?
54async fn process_commands(store_path: &str, graph: &str) -> io::Result<()> {
55 let store = open_directory_store(store_path);
56 let graph = store
57 .open(graph)
58 .await?
59 .expect(&format!("expected graph {} to exist", graph));
60
61 // There are two types of builders. One creates a new base layer,
62 // which has no parent. The other creates a child layer, which has
63 // another layer as its parent.
64 let builder = match graph.head().await? {
65 Some(layer) => layer.open_write().await?,
66 None => store.create_base_layer().await?,
67 };
68 let mut stdin = io::BufReader::new(io::stdin()).lines();
69
70 while let Some(line) = stdin.next_line().await? {
71 let segment = line.trim();
72 if segment.len() == 0 {
73 continue;
74 }
75
76 let command = parse_command(segment).await?;
77
78 // add all the input data into the builder.
79 // The builder keeps an in-memory list of added and removed
80 // triples. If the same triple is added and removed on the
81 // same builder, it is a no-op. This is even the case when it
82 // is then later re-added on the same builder.
83 //
84 // Since no io is happening, adding triples to the builder is
85 // not a future.
86 match command {
87 Command::Add(triple) => builder.add_value_triple(triple)?,
88 Command::Remove(triple) => builder.remove_value_triple(triple)?,
89 }
90 }
91
92 // When commit is called, the builder writes its data to
93 // persistent storage.
94 let layer = builder.commit().await?;
95
96 // While a layer exists now, it's not yet attached to anything,
97 // and is therefore unusable unless you know the exact identifier
98 // of the layer itself. To make this the graph data, we have to
99 // set the grap head to this layer.
100 graph.set_head(&layer).await?;
101
102 println!(
103 "Added: {}, removed: {}",
104 layer.triple_layer_addition_count().await?,
105 layer.triple_layer_removal_count().await?
106 );
107
108 Ok(())
109}Trait Implementations
Sourceimpl Clone for StoreLayer
impl Clone for StoreLayer
Sourcefn clone(&self) -> StoreLayer
fn clone(&self) -> StoreLayer
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreimpl Eq for StoreLayer
Sourceimpl Layer for StoreLayer
impl Layer for StoreLayer
fn parent_name(&self) -> Option<[u32; 5]>
Sourcefn node_and_value_count(&self) -> usize
fn node_and_value_count(&self) -> usize
Sourcefn predicate_count(&self) -> usize
fn predicate_count(&self) -> usize
Sourcefn subject_id(&self, subject: &str) -> Option<u64>
fn subject_id(&self, subject: &str) -> Option<u64>
Sourcefn predicate_id(&self, predicate: &str) -> Option<u64>
fn predicate_id(&self, predicate: &str) -> Option<u64>
Sourcefn object_node_id(&self, object: &str) -> Option<u64>
fn object_node_id(&self, object: &str) -> Option<u64>
Sourcefn object_value_id(&self, object: &TypedDictEntry) -> Option<u64>
fn object_value_id(&self, object: &TypedDictEntry) -> Option<u64>
Sourcefn id_subject(&self, id: u64) -> Option<String>
fn id_subject(&self, id: u64) -> Option<String>
Sourcefn id_predicate(&self, id: u64) -> Option<String>
fn id_predicate(&self, id: u64) -> Option<String>
Sourcefn id_object(&self, id: u64) -> Option<ObjectType>
fn id_object(&self, id: u64) -> Option<ObjectType>
Sourcefn id_object_is_node(&self, id: u64) -> Option<bool>
fn id_object_is_node(&self, id: u64) -> Option<bool>
Sourcefn triple_exists(&self, subject: u64, predicate: u64, object: u64) -> bool
fn triple_exists(&self, subject: u64, predicate: u64, object: u64) -> bool
Sourcefn triples(&self) -> Box<dyn Iterator<Item = IdTriple> + Send>
fn triples(&self) -> Box<dyn Iterator<Item = IdTriple> + Send>
fn triples_s(&self, subject: u64) -> Box<dyn Iterator<Item = IdTriple> + Send>
fn triples_sp( &self, subject: u64, predicate: u64, ) -> Box<dyn Iterator<Item = IdTriple> + Send>
fn triples_p(&self, predicate: u64) -> Box<dyn Iterator<Item = IdTriple> + Send>
fn triples_o(&self, object: u64) -> Box<dyn Iterator<Item = IdTriple> + Send>
Sourcefn clone_boxed(&self) -> Box<dyn Layer>
fn clone_boxed(&self) -> Box<dyn Layer>
Sourcefn triple_addition_count(&self) -> usize
fn triple_addition_count(&self) -> usize
Sourcefn triple_removal_count(&self) -> usize
fn triple_removal_count(&self) -> usize
Sourcefn all_counts(&self) -> LayerCounts
fn all_counts(&self) -> LayerCounts
fn single_triple_sp(&self, subject: u64, predicate: u64) -> Option<IdTriple>
Sourcefn id_object_node(&self, id: u64) -> Option<String>
fn id_object_node(&self, id: u64) -> Option<String>
Sourcefn id_object_value(&self, id: u64) -> Option<TypedDictEntry>
fn id_object_value(&self, id: u64) -> Option<TypedDictEntry>
Sourcefn id_object_is_value(&self, id: u64) -> Option<bool>
fn id_object_is_value(&self, id: u64) -> Option<bool>
Sourcefn id_triple_exists(&self, triple: IdTriple) -> bool
fn id_triple_exists(&self, triple: IdTriple) -> bool
Sourcefn value_triple_exists(&self, triple: &ValueTriple) -> bool
fn value_triple_exists(&self, triple: &ValueTriple) -> bool
Sourcefn value_triple_to_id(&self, triple: &ValueTriple) -> Option<IdTriple>
fn value_triple_to_id(&self, triple: &ValueTriple) -> Option<IdTriple>
ValueTriple to an IdTriple, returning None if any of the strings in the triple could not be resolved.Sourcefn value_triple_to_partially_resolved(
&self,
triple: ValueTriple,
) -> PartiallyResolvedTriple
fn value_triple_to_partially_resolved( &self, triple: ValueTriple, ) -> PartiallyResolvedTriple
Sourcefn id_triple_to_string(&self, triple: &IdTriple) -> Option<ValueTriple>
fn id_triple_to_string(&self, triple: &IdTriple) -> Option<ValueTriple>
Sourcefn triple_count(&self) -> usize
fn triple_count(&self) -> usize
Sourceimpl PartialEq for StoreLayer
impl PartialEq for StoreLayer
Sourcefn eq(&self, other: &StoreLayer) -> bool
fn eq(&self, other: &StoreLayer) -> bool
self and other values to be equal, and is used by ==.Auto Trait Implementations
impl !RefUnwindSafe for StoreLayer
impl !UnwindSafe for StoreLayer
impl Freeze for StoreLayer
impl Send for StoreLayer
impl Sync for StoreLayer
impl Unpin for StoreLayer
impl UnsafeUnpin for StoreLayer